from django.contrib.auth import get_user_model
from django.urls import reverse
from django.test import TestCase
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient

User = get_user_model()


class NotificationTestCase(TestCase):
    fixtures = ['users', 'notificationreceipts']

    def setUp(self):
        self.client = APIClient()

    def authenticate(self, user=None):
        self.client.credentials()
        if user:
            token, created = Token.objects.get_or_create(user=user)
            self.client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)

    def list(self, **kwargs):
        return self.client.get(
            reverse('notificationreceipts_list'),
            format='json',
            **kwargs
        )

    @staticmethod
    def get_default_user():
        return User.objects.get(id=1)


class NotificationReceiptListTestCase(NotificationTestCase):
    def setUp(self):
        super(NotificationReceiptListTestCase, self).setUp()
        self.user = self.get_default_user()

    def test_list_response(self):
        self.authenticate(user=self.user)
        response = self.list()
        self.assertEqual(response.status_code, status.HTTP_200_OK)



from django.contrib.auth import get_user_model
from django.db.models import QuerySet
from push_notifications.models import APNSDevice, GCMDevice
from .base import NotificationBase
from .models import NotificationReceipt

User = get_user_model()


class NotificationClient(object):
    def get_devices(self, users):
        return APNSDevice.objects.filter(user__in=users), GCMDevice.objects.filter(user__in=users)

    def create_receipt(self, notification, user):
        return NotificationReceipt(payload=notification, user=user)

    def send_notification(self, notification, users):
        assert isinstance(notification, NotificationBase)
        assert isinstance(users, QuerySet)
        assert users.model is User

        apns_devices, gcm_devices = self.get_devices(users)
        apns_devices.send_message(notification.message, kwargs=notification.additional_data)
        gcm_devices.send_message(notification.message, kwargs=notification.additional_data)

        NotificationReceipt.objects.bulk_create(
            [self.create_receipt(notification=notification, user=user) for user in users]
        )



from django.conf.urls import url
from push_notifications.api.rest_framework import APNSDeviceAuthorizedViewSet, GCMDeviceAuthorizedViewSet
from rest_framework.urlpatterns import format_suffix_patterns
from .views import NotificationReceiptListCreateView

urlpatterns = [
    url(r'^notifications$', NotificationReceiptListCreateView.as_view(), name='notificationreceipts_list'),
    url(r'^device/apns/?$', APNSDeviceAuthorizedViewSet.as_view({'post': 'create'}), name='create_apns_device'),
    url(r'^device/gcm/?$', GCMDeviceAuthorizedViewSet.as_view({'post': 'create'}), name='create_gcm_device'),
]

urlpatterns = format_suffix_patterns(urlpatterns)
